<?php
namespace Drupal\fullcalendar_block\Plugin\Block;

use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a FullCalendar Block.
 *
 * @Block(
 *   id = "fullcalendar_block",
 *   admin_label = @Translation("FullCalendar block"),
 *   category = @Translation("Calendar"),
 * )
 */
class FullCalendarBlock extends BlockBase {

  protected  static $blockIndex = 0;

  /**
   * {@inheritdoc}
   */
  public function build() {
    $block_index = self::$blockIndex++;

    $config = $this->getConfiguration();
    $event_url = $config['event_source'] ?? '';
    $initial_view = $config['initial_view'] ?? 'dayGridMonth';
    $header_start = $config['header_start'] ?? 'prev,next today';
    $header_center = $config['header_center'] ?? 'title';
    $header_end = $config['header_end'] ?? 'timeGridWeek,timeGridDay';
    $dialog_open = $config['open_dialog'] ?? 1;
    $dialog_width = $config['dialog_width'] ?? 800;
    $advanced_settings = $config['advanced'] ?? '';

    // Fullcalendar options.
    $calendar_options = [
      'initialView' => $initial_view,
      'events' => $event_url,
      'headerToolbar' => [
        'start' => $header_start,
        'center' => $header_center,
        'end' => $header_end,
      ],
    ];

    if (!empty($advanced_settings)) {
      $calendar_options = array_merge($calendar_options, json_decode($advanced_settings, true));
    }

    $block_content = [
      '#theme' =>  'fullcalendar_block',
      '#block_index' => $block_index,
      '#attached' => [
        'library' => [
          'fullcalendar_block/fullcalendar',
        ],
      ],
    ];

    $block_content['#attached']['drupalSettings']['fullCalendarBlock'][$block_index] = [
      'calendar_options' => json_encode($calendar_options),
      'dialog_open' => $dialog_open,
      'dialog_width' => $dialog_width,
    ];

    return $block_content;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['event_source'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event source URL'),
      '#description' => $this->t('The URL where the calendar events data feeds'),
      '#default_value' => $config['event_source'] ?? '',
    ];

    $form['initial_view'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Initial View'),
      '#description' => $this->t('The initial view of the calendar'),
      '#default_value' => $config['initial_view'] ?? 'dayGridMonth',
    ];

    // Header toolbar settings.
    $form['header_toolbar'] = [
      '#type' => 'details',
      '#title' => $this->t('Header Toolbar'),
      '#description' => $this->t('The header toolbar settings.'),
    ];

    $form['header_toolbar']['header_start'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Start of the header toolbar'),
      '#description' => $this->t('Start area will normally be on the left. if RTL, will be on the right'),
      '#default_value' => $config['header_start'] ?? 'prev,next today',
    ];

    $form['header_toolbar']['header_center'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Center of the header toolbar'),
      '#description' => $this->t('The default value is title if leave this empty'),
      '#default_value' => $config['header_center'] ?? 'title',
    ];

    $form['header_toolbar']['header_end'] = [
      '#type' => 'textfield',
      '#title' => $this->t('End of the header toolbar'),
      '#description' => $this->t('The default value is Week and Day view if leave this empty'),
      '#default_value' => $config['header_end'] ?? 'timeGridWeek,timeGridDay',
    ];

    // Click event settings.
    $form['click_event'] = [
      '#type' => 'details',
      '#title' => $this->t('Click event settings.'),
    ];

    $form['click_event']['open_dialog'] = [
      '#type' => 'radios',
      '#title' => $this->t('Click on an event'),
      '#default_value' => 1,
      '#options' => [
        0 => $this->t('Open in a new tab'),
        1 => $this->t('Open in a dialog'),
      ],
      '#default_value' => $config['open_dialog'] ?? 1,
      '#attributes' => [
        //define static data condition attribute so we can easier select it
        'data-condition' => 'field-open-dialog',
      ],
    ];

    $form['click_event']['dialog_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dialog width'),
      '#default_value' => $config['dialog_width'] ?? 800,
      '#size' => '5',
      '#states' => [
        //show this textfield only if 'open in a dialog' is selected above
        'visible' => [
          ':input[data-condition="field-open-dialog"]' => ['value' => 1],
        ],
      ],
    ];

    // Advanced settings.
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced settings.'),
    ];

    $form['advanced']['addition'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Advanced settings in JSON format.'),
      '#default_value' => $config['advanced'] ?? '',
      '#description' => $this->t('It must be in valid JSON format.<br/>See %doc for available options.', array(
        '%doc' => Link::fromTextAndUrl($this->t('Fullcalendar Document'), Url::fromUri('https://fullcalendar.io/docs#toc', array('attributes' => array('target' => '_blank'))))->toString(),
      )),

    ];


    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['event_source'] = $values['event_source'];
    $this->configuration['initial_view'] = $values['initial_view'];
    $this->configuration['header_start'] = $values['header_toolbar']['header_start'];
    $this->configuration['header_center'] = $values['header_toolbar']['header_center'];
    $this->configuration['header_end'] = $values['header_toolbar']['header_end'];
    $this->configuration['open_dialog'] = $values['click_event']['open_dialog'];
    $this->configuration['dialog_width'] = $values['click_event']['dialog_width'];
    $this->configuration['advanced'] = $values['advanced']['addition'];
  }

}